package com.zoniot.ccrc.controller.system;


import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dto.BuildingDto;
import com.zoniot.ccrc.dto.BuildingSelectDto;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.entity.Building;
import com.zoniot.ccrc.service.BuildingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;

@Controller
@RequestMapping("system/building")
@Api(tags = "建筑管理")
public class BuildingController {

    @Autowired
    private BuildingService buildingService;


    @ResponseBody
    @GetMapping("/select")
    @ApiOperation(value = "获取建筑树形列表(用户权限)")
    public AjaxMessage<List<BuildingSelectDto>> select(
            @ApiParam(value = "场景id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "名称", required = false) @RequestParam(required = false) Integer name
    ) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<BuildingSelectDto> list = buildingService.select(loginUser, sysId);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }




    @ResponseBody
    @GetMapping("/area")
    @ApiOperation(value = "获取区域树形列表")
    public AjaxMessage<List<BuildingSelectDto>> area(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId
    ) {
        List<BuildingSelectDto> list = buildingService.area(sysId);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }





    @ResponseBody
    @GetMapping("/getBuildingPage")
    @ApiOperation(value = "查询建筑分页")
    public AjaxMessage<Pagination<BuildingDto>> getBuildingPage(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "建筑名称", required = false) @RequestParam(required = false) String buildingName,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区", required = false) @RequestParam(required = false) Integer community,
            @ApiParam(value = "页数,非必传，默认第一页", required = false, defaultValue = "1") @RequestParam(required = false, defaultValue = "1") int pageNum,
            @ApiParam(value = "条数，非必传，默认15条", required = false, defaultValue = "15") @RequestParam(required = false, defaultValue = "15") int pageSize
    ) {
        Pagination<BuildingDto> pageInfo = buildingService.getBuildingPage(sysId, StringUtils.trim(buildingName), province, city, region, community, pageNum, pageSize);
        return new AjaxMessage<>(ResultStatus.OK, pageInfo);
    }



    @ResponseBody
    @GetMapping("/getBuilding")
    @ApiOperation(value = "查询建筑详情")
    public AjaxMessage<BuildingDto> getBuilding(
            @ApiParam(value = "建筑id", required = true) @RequestParam(required = true) Integer buildingId
    ) {
        BuildingDto building = buildingService.getBuilding(buildingId);
        return new AjaxMessage<>(ResultStatus.OK, building);
    }

    @ResponseBody
    @PostMapping("/add")
    //@PreAuthorize("hasAuthority('sys:building:add')")
    @ApiOperation(value = "添加建筑", notes = "权限：sys:building:add")
    public AjaxMessage add(
            @ApiParam(value = "建筑", required = true) @RequestBody(required = true) Building building
    ) {
        buildingService.add(building);
        return new AjaxMessage<>(ResultStatus.OK);
    }

    @ResponseBody
    @PutMapping("/edit")
    //@PreAuthorize("hasAuthority('sys:building:edit')")
    @ApiOperation(value = "添加建筑", notes = "权限：sys:building:edit")
    public AjaxMessage edit(
            @ApiParam(value = "建筑", required = true) @RequestBody(required = true) Building building
    ) {
        buildingService.edit(building);
        return new AjaxMessage<>(ResultStatus.OK);
    }

    @ResponseBody
    @DeleteMapping("/del")
    //@PreAuthorize("hasAuthority('sys:building:del')")
    @ApiOperation(value = "删除建筑", notes = "权限：sys:building:del")
    public AjaxMessage del(
            @ApiParam(value = "建筑id", required = true) @RequestParam(required = true) Integer buildingId
    ) {
        buildingService.del(buildingId);
        return new AjaxMessage<>(ResultStatus.OK);
    }




}
