package com.zoniot.ccrc.controller.system;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.dto.BuildingSelectDto;
import com.zoniot.ccrc.dto.DeviceDto;
import com.zoniot.ccrc.service.DeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Controller
@RequestMapping("device")
@Api(tags = "设备管理")
public class DeviceController {
    @Autowired
    private DeviceService deviceService;

    @ResponseBody
    @GetMapping("/pageList")
    @ApiOperation(value = "获取设备分页")
    public AjaxMessage<Pagination<DeviceDto>> devicePageList(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型id", required = false) @RequestParam(required = false) Integer deviceTypeId,
            @ApiParam(value = "机构id", required = false) @RequestParam(required = false) Integer orgId,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区id", required = false) @RequestParam(required = false) Integer communityId,
            @ApiParam(value = "建筑id", required = false) @RequestParam(required = false) Integer buildingId,
            @ApiParam(value = "设备编号", required = false) @RequestParam(required = false) String deviceNo,
            @ApiParam(value = "客户名称", required = false) @RequestParam(required = false) String clientName,
            @ApiParam(value = "位置描述", required = false) @RequestParam(required = false) String locDesc,
            @ApiParam(value = "设备状态  1:正常 2:故障 3:无 4: 预警 5:未启用", required = false) @RequestParam(required = false) Integer status,
            @ApiParam(value = "阀门状态 0：关阀 1：开阀  2：无阀", required = false) @RequestParam(required = false) Integer valveStatus,
            @ApiParam(value = "排序参数，对应列表字段", required = false) @RequestParam(required = false,defaultValue = "sd.date_create") String sortColumn,
            @ApiParam(value = "排序方式，顺序：ASC  倒序：DESC", required = false) @RequestParam(required = false, defaultValue = "DESC") String sortOrder,
            @ApiParam(value = "页数,非必传，默认第一页", required = false, defaultValue = "1") @RequestParam(required = false, defaultValue = "1") int pageNum,
            @ApiParam(value = "条数，非必传，默认15条", required = false, defaultValue = "15") @RequestParam(required = false, defaultValue = "15") int pageSize
    ) {
        Pagination<DeviceDto> pageInfo = deviceService.pageList(sysId,deviceTypeId,orgId,province,city,region,communityId,buildingId,deviceNo,clientName,locDesc,status,valveStatus, sortColumn, sortOrder, pageNum, pageSize);
        return new AjaxMessage<>(ResultStatus.OK, pageInfo);
    }


    @GetMapping("/getDeviceListExcel")
    @ApiOperation(value = "导出设备列表excel")
    public void getDeviceListExcel(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型id", required = false) @RequestParam(required = false) Integer deviceTypeId,
            @ApiParam(value = "机构id", required = false) @RequestParam(required = false) Integer orgId,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区id", required = false) @RequestParam(required = false) Integer communityId,
            @ApiParam(value = "建筑id", required = false) @RequestParam(required = false) Integer buildingId,
            @ApiParam(value = "设备编号", required = false) @RequestParam(required = false) String deviceNo,
            @ApiParam(value = "客户名称", required = false) @RequestParam(required = false) String clientName,
            @ApiParam(value = "位置描述", required = false) @RequestParam(required = false) String locDesc,
            @ApiParam(value = "设备状态  1:正常 2:故障 3:无 4: 预警 5:未启用", required = false) @RequestParam(required = false) Integer status,
            @ApiParam(value = "阀门状态 0：关阀 1：开阀  2：无阀", required = false) @RequestParam(required = false) Integer valveStatus,
            @ApiParam(value = "排序参数，对应列表字段", required = false) @RequestParam(required = false,defaultValue = "sd.date_create") String sortColumn,
            @ApiParam(value = "排序方式，顺序：ASC  倒序：DESC", required = false) @RequestParam(required = false,defaultValue = "DESC") String sortOrder,
            HttpServletResponse httpServletResponse
    ) {
        deviceService.getDeviceListExcel(sysId,deviceTypeId,orgId,province,city,region,communityId,buildingId,deviceNo,clientName,locDesc,status,valveStatus, sortColumn, sortOrder, httpServletResponse);

    }


    @ResponseBody
    @GetMapping("/deviceAreaList")
    @ApiOperation(value = "获取设备数量区域列表")
    public AjaxMessage<List<BuildingSelectDto>> deviceAreaList(
            @ApiParam(value = "系统id", required = false) @RequestParam(required = false) Integer sysId,
            @ApiParam(value = "设备类型id", required = false) @RequestParam(required = false) Integer deviceTypeId,
            @ApiParam(value = "机构id", required = false) @RequestParam(required = false) Integer orgId,
            @ApiParam(value = "省", required = false) @RequestParam(required = false) Integer province,
            @ApiParam(value = "市", required = false) @RequestParam(required = false) Integer city,
            @ApiParam(value = "区", required = false) @RequestParam(required = false) Integer region,
            @ApiParam(value = "小区id", required = false) @RequestParam(required = false) Integer communityId,
            @ApiParam(value = "建筑id", required = false) @RequestParam(required = false) Integer buildingId,
            @ApiParam(value = "设备编号", required = false) @RequestParam(required = false) String deviceNo,
            @ApiParam(value = "客户名称", required = false) @RequestParam(required = false) String clientName,
            @ApiParam(value = "位置描述", required = false) @RequestParam(required = false) String locDesc,
            @ApiParam(value = "设备状态  1:正常 2:故障 3:无 4: 预警 5:未启用", required = false) @RequestParam(required = false) Integer status,
            @ApiParam(value = "阀门状态 0：关阀 1：开阀  2：无阀", required = false) @RequestParam(required = false) Integer valveStatus
    ) {
        List<BuildingSelectDto> list = deviceService.deviceAreaList(sysId,deviceTypeId,orgId,province,city,region,communityId,buildingId,deviceNo,clientName,locDesc,status,valveStatus);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }
}
