package com.zoniot.ccrc.controller.system;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.dto.OrganizationDto;
import com.zoniot.ccrc.entity.Organization;
import com.zoniot.ccrc.service.OrganizationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Controller
@ResponseBody
@RequestMapping("system/organ")
@Api(tags = "机构管理")
public class OrganController {
    @Autowired
    private OrganizationService organizationService;


    @GetMapping("getOrganTree")
    @ApiOperation(value = "获取机构树", notes = "权限：sys:organ:query")
    //@PreAuthorize("hasAuthority('sys:organ:query')")
    public AjaxMessage<List<OrganizationDto>> getOrganTree(
    ) {

        List<OrganizationDto> list = organizationService.getOrganTree();
        return new AjaxMessage<>(ResultStatus.OK, list);
    }

    @PostMapping("add")
    //@PreAuthorize("hasAuthority('sys:organ:add')")
    @ApiOperation(value = "添加机构", notes = "权限：sys:organ:add")
    public AjaxMessage addRole(
            @ApiParam(value = "机构", required = true) @RequestBody(required = true) Organization organization
    ) {

        organizationService.add(organization);
        return new AjaxMessage(ResultStatus.OK);
    }

    @PutMapping("edit")
    //@PreAuthorize("hasAuthority('sys:organ:edit')")
    @ApiOperation(value = "编辑机构", notes = "权限：sys:organ:edit")
    public AjaxMessage editRole(
            @ApiParam(value = "机构", required = true) @RequestBody(required = true) Organization organization
    ) {
        organizationService.edit(organization);
        return new AjaxMessage(ResultStatus.OK);
    }

    @DeleteMapping("delete")
    //@PreAuthorize("hasAuthority('sys:organ:del')")
    @ApiOperation(value = "删除机构", notes = "权限：sys:organ:del")
    public AjaxMessage delRole(
            @ApiParam(value = "机构id", required = true) @RequestParam(required = true) Integer id
    ) {
        organizationService.delete(id);
        return new AjaxMessage(ResultStatus.OK);
    }

    @GetMapping("deleteQuery")
    @ApiOperation(value = "删除查询", notes = "删除查询,返回用户个数个子类个数")
    public AjaxMessage deleteQuery(
            @ApiParam(value = "机构id", required = true) @RequestParam(required = true) Integer id
    ) {
        Map<String,Integer> map =  organizationService.deleteQuery(id);
        return new AjaxMessage<>(ResultStatus.OK, map);
    }

    @GetMapping("getOrganTreeForCustomer")
    @ApiOperation(value = "获取机构树(数据权限)", notes = "权限：sys:organ:query")
    //@PreAuthorize("hasAuthority('sys:organ:query')")
    public AjaxMessage<List<OrganizationDto>> getOrganTreeForCustomer(
    ) {

        List<OrganizationDto> list = organizationService.findListByCustomer();
        return new AjaxMessage<>(ResultStatus.OK, list);
    }
}
