package com.zoniot.ccrc.controller.system;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.dto.RoleDto;
import com.zoniot.ccrc.entity.Permission;
import com.zoniot.ccrc.entity.Role;
import com.zoniot.ccrc.service.PermissionService;
import com.zoniot.ccrc.service.RoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Controller
@ResponseBody
@RequestMapping("system/role")
@Api(description = "角色管理")
public class RoleManageController {


    @Autowired
    private PermissionService permissionService;
    @Autowired
    private RoleService roleService;

    @GetMapping("getRoleList")
    @ApiOperation(value = "获取角色列表", notes = "权限：sys:role:query")
    @PreAuthorize("hasAuthority('sys:role:query')")
    public AjaxMessage<Pagination<Role>> getRoleList(
            @ApiParam(value = "名称", required = false) @RequestParam(required = false) String name,
            @ApiParam(value = "页数,非必传，默认第一页", required = false, defaultValue = "1") @RequestParam(required = false, defaultValue = "1") int pageNum,
            @ApiParam(value = "条数，非必传，默认15条", required = false, defaultValue = "15") @RequestParam(required = false, defaultValue = "15") int pageSize
    ) {
        if (name != null) {
            name = name.trim();
        }
        Pagination<Role> pageInfo = roleService.getRoleList(name, pageNum, pageSize);
        return new AjaxMessage<>(ResultStatus.OK, pageInfo);
    }

    @PostMapping("add")
    @PreAuthorize("hasAuthority('sys:role:add')")
    @ApiOperation(value = "添加角色", notes = "权限：sys:role:add")
    public AjaxMessage addRole(
            @ApiParam(value = "角色", required = true) @RequestBody(required = true) @Validated RoleDto roleDto
    ) {
        roleService.addRole(roleDto);
        return new AjaxMessage(ResultStatus.OK);
    }

    @PutMapping("edit")
    @PreAuthorize("hasAuthority('sys:role:edit')")
    @ApiOperation(value = "编辑角色", notes = "权限：sys:role:edit")
    public AjaxMessage editRole(
            @ApiParam(value = "角色", required = true) @RequestBody(required = true) Role role
    ) {
        roleService.editRole(role);
        return new AjaxMessage(ResultStatus.OK);
    }

    @DeleteMapping("delete")
    @PreAuthorize("hasAuthority('sys:role:del')")
    @ApiOperation(value = "删除角色", notes = "权限：sys:role:del")
    public AjaxMessage delRole(
            @ApiParam(value = "角色id", required = true) @RequestParam(required = true) Integer roleId
    ) {
        roleService.delRole(roleId);
        return new AjaxMessage(ResultStatus.OK);
    }

    @GetMapping("getRolePermission")
    @ApiOperation(value = "获取岗位权限")
    public AjaxMessage<List<Permission>> getRolePermission(
            @ApiParam(value = "角色id", required = true) @RequestParam(required = true) Integer roleId
    ) {
        List<Permission> list = permissionService.getRolePermissionByRoleId(roleId);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }


    @GetMapping("getAllPermission")
    @ApiOperation(value = "获取全部权限")
    public AjaxMessage<List<Permission>> getAllPermission() {
        List<Permission> list = permissionService.getAllPermission();
        return new AjaxMessage<>(ResultStatus.OK, TreeUtil.getMenuTree(list, 1, 1));
    }


    @PostMapping("savePermission")
    @ApiOperation(value = "更改权限")
    public AjaxMessage savePermission(
            @ApiParam(value = "角色id", required = true) @RequestParam(required = true) Integer roleId,
            @RequestParam(required = true) @ApiParam(value = "权限ids 多个英文逗号隔开", required = true) List<Integer> pids
    ) {
        permissionService.savePermission(roleId, pids);
        return new AjaxMessage(ResultStatus.OK);
    }
}
