package com.zoniot.ccrc.controller.system;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.dto.SiteDto;
import com.zoniot.ccrc.dto.UserDto;
import com.zoniot.ccrc.entity.Permission;
import com.zoniot.ccrc.service.PermissionService;
import com.zoniot.ccrc.service.SiteService;
import com.zoniot.ccrc.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Controller
@ResponseBody
@RequestMapping("system/tenant")
@Api(tags = "站点管理")
public class SiteController {

    @Autowired
    private SiteService siteService;
    @Autowired
    private UserService userService;
    @Autowired
    private PermissionService permissionService;




    @GetMapping("unbindUser")
    @ApiOperation(value = "获取未绑定用户")
    public AjaxMessage<List<UserDto>> unbindUser(
            @ApiParam(value = "站点id", required = false) @RequestParam(required = false) Integer siteId
    ) {
        List<UserDto> list = userService.unbindUser(siteId);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }

    @GetMapping("pageList")
    @PreAuthorize("hasAuthority('sys:site:query')")
    @ApiOperation(value = "获取站点列表", notes = "权限：sys:site:query")
    public AjaxMessage<Pagination<SiteDto>> getPage(
            @ApiParam(value = "站点id, 多个逗号隔开", required = false) @RequestParam(required = false) List<Integer> siteIds,
            @ApiParam(value = "站点名称", required = false) @RequestParam(required = false) String name,
            @ApiParam(value = "页数,非必传，默认第一页", required = false, defaultValue = "1") @RequestParam(required = false, defaultValue = "1") int pageNum,
            @ApiParam(value = "条数，非必传，默认15条", required = false, defaultValue = "15") @RequestParam(required = false, defaultValue = "9") int pageSize
    ) {
        Pagination<SiteDto> pageInfo = siteService.getPage(siteIds, StringUtils.trim(name), pageNum, pageSize);
        return new AjaxMessage<>(ResultStatus.OK, pageInfo);
    }


    @GetMapping("getList")
    @ApiOperation(value = "获取全部站点列表")
    public AjaxMessage<List<SiteDto>> getList(
    ) {
        List<SiteDto> pageInfo = siteService.getList();
        return new AjaxMessage<>(ResultStatus.OK, pageInfo);
    }



    @PostMapping("add")
    @PreAuthorize("hasAuthority('sys:site:add')")
    @ApiOperation(value = "添加站点", notes = "权限：sys:site:add")
    public AjaxMessage add(
            @ApiParam(value = "站点", required = true) @RequestBody(required = true) SiteDto siteDto
    ) {
        siteService.add(siteDto);
        return new AjaxMessage(ResultStatus.OK);
    }

    @PutMapping("edit")
    @ApiOperation(value = "编辑站点", notes = "权限：sys:site:edit")
    @PreAuthorize("hasAuthority('sys:site:edit')")
    public AjaxMessage edit(
            @ApiParam(value = "站点", required = true) @RequestBody(required = true) SiteDto siteDto
    ) {
        siteService.edit(siteDto);
        return new AjaxMessage(ResultStatus.OK);
    }

    @DeleteMapping("/del")
    @PreAuthorize("hasAuthority('sys:site:del')")
    @ApiOperation(value = "删除站点", notes = "权限：sys:site:del")
    public AjaxMessage del(
            @ApiParam(value = "站点id", required = true) @RequestParam(required = true) Integer id
    ) {
        siteService.delete(id);
        return new AjaxMessage(ResultStatus.OK);
    }

    @GetMapping("getPermissionBySiteType")
    @ApiOperation(value = "根据站点类型获取权限")
    public AjaxMessage<List<Permission>> getPermissionBySiteType(
            @ApiParam(value = "站点类型 1:默认", required = true) @RequestParam(required = true) Integer type
    ) {
        List<Permission> list = permissionService.getPermissionBySiteType(type);
        return new AjaxMessage<>(ResultStatus.OK, TreeUtil.getMenuTree(list, 1, 1));
    }



    @GetMapping("getSitePermission")
    @ApiOperation(value = "获取站点拥有的功能权限")
    public AjaxMessage<List<Permission>> getSitePermission(
            @ApiParam(value = "站点id", required = true) @RequestParam(required = true) Integer siteId
    ) {
        List<Permission> list = permissionService.getSitePermission(siteId);
        return new AjaxMessage<>(ResultStatus.OK, list);
    }
}
