package com.zoniot.ccrc.repository;

import lombok.Data;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * <p>抄表记录</p>
 * sh.enableSharding("meter-reading-database");
 * sh.shardCollection("meter-reading-database.sc_meter_read_record",{"readDate":1,"deviceId":1});
 * db.sc_meter_read_record.createIndex({readDate:1,customerId:1,buildingId:1,siteId:1,sysId:1,readStatus:1,concentratorId:1},{background: true,name:'idx_1'})
 * @Author wilian.peng
 * @Date 2020/12/22 16:59
 * @Version 1.0
 */
@Data
@Document(collection = "sc_meter_read_record")
public class MongoMeterReadRecord implements Serializable {
    /**
    * 主键
    */
    private Long id;

    /**
    * 读表日期
    */
    private Integer readDate;

    /**
     * 站点
     */
    private Integer siteId;

    /**
     * 场景
     */
    private Integer sysId;

    /**
    * 省
    */
    private Integer province;

    /**
    * 市
    */
    private Integer city;

    /**
    * 区
    */
    private Integer region;

    /**
    * 小区
    */
    private Integer community;

    /**
    * 客户
    */
    private Integer customerId;

    /**
    * 集中器
    */
    private Integer concentratorId;

    /**
    * 采集器
    */
    private Integer collectorId;

    /**
    * 建筑
    */
    private Integer buildingId;

    /**
    * 安装地址
    */
    private String location;

    /**
    * 设备类型
    */
    private Integer deviceTypeId;

    /**
    * 设备id
    */
    private Long deviceId;

    /**
    * 节点编号
    */
    private String deviceNo;

    /**
    * 电子号
    */
    private String meterNo;

    /**
    * 档案号
    */
    private String meterFileNo;

    /**
    * 读表时间
    */
    private Date readTime;

    /**
    * 读表状态
    */
    private String readStatus;

    /**
    * 读表数据
    */
    private String readData;

    /**
    * 最近有效数据
    */
    private String lastValid;

    /**
    * 距离上次的消耗
    */
    private BigDecimal lastCost;

    /**
    * 状态
    */
    private Integer status;

    /**
    * 创建时间
    */
    private Date dateCreate;

    /**
    * 更新时间
    */
    private Date dateUpdate;

    /**
    * 创建人
    */
    private String createBy;

    /**
    * 更新人
    */
    private String updateBy;

    /***********************************以下字段将存入MongoDB中**************************************/
    private String communityName;

    private String buildingName ;

    private String customerName ;

    private String provinceName ;

    private String cityName ;

    private String regionName ;

    private String deviceTypeName ;

    private String concentratorNo ;

    private String collectorNo ;

    private Integer manufacturerId ;

    private String manufacturerName ;

    private Integer channelNumberId;

    private String channelName ;

    /**
     * 最后上送数据,格式如下：
     * {"数据ID"：11111，data:{"WSV":"1","VOL":"2.5"}}
     */
    private Map<String, String> lastSendData ;
}