package com.zoniot.ccrc.scheduled;



import com.zoniot.ccrc.dao.WarningLogMapper;
import com.zoniot.ccrc.dao.WarningMessageMapper;
import com.zoniot.ccrc.dao.WarningRuleMapper;
import com.zoniot.ccrc.entity.WarningLog;
import com.zoniot.ccrc.entity.WarningMessage;
import com.zoniot.ccrc.entity.WarningRule;
import com.zoniot.ccrc.service.MongoMeterReadRecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

@Slf4j
@Component
@Configuration
@EnableScheduling
public class WarningMessageJob {


    @Resource
    private WarningRuleMapper warningRuleMapper;
    @Resource
    private WarningLogMapper warningLogMapper;
    @Resource
    private WarningMessageMapper warningMessageMapper;
    @Autowired
    private MongoMeterReadRecordService mongoMeterReadRecordService;


    /**
     * 每天预警消息
     */
    @Scheduled(cron = "0 0 10 * * ?")
    public void warningMessage() {
        log.info("begin warningMessage job");
        List<WarningRule> list = warningRuleMapper.getList(null);
        if (list != null && list.size() > 0) {
            for (WarningRule warningRule : list) {
                    Boolean flag =  mongoMeterReadRecordService.complyWithWarningRules(warningRule.getDeviceId(),warningRule.getWarningType());
                    if ( flag) {
                        addMessageInfo(warningRule,"单日用水量超过2吨");
                }


            }
        }
        log.info("end warningMessage job");
    }
    private void addMessageInfo(WarningRule warningRule,String content){
        WarningLog warningLogNew = new WarningLog();

        warningLogNew.setDeviceId(warningRule.getDeviceId());
        warningLogNew.setWarningType(warningRule.getWarningType());
        warningLogNew.setFeedbackStatus(0);
        warningLogNew.setFeedbackContent("");
        //warningLogNew.setLastDate(LocalDateTime.now());
        warningLogNew.setStatus(1);
        warningLogNew.setCreateBy("system");
        warningLogNew.setDateCreate(LocalDateTime.now());
        warningLogNew.setUpdateBy("system");
        warningLogNew.setDateUpdate(LocalDateTime.now());
        warningLogMapper.insertSelective(warningLogNew);

        WarningMessage warningMessageNew = new WarningMessage();
        warningMessageNew.setWarningLogId(warningLogNew.getId());
        warningMessageNew.setContent(content);
        warningMessageNew.setStatus(1);
        warningMessageNew.setCreateBy("system");
        warningMessageNew.setDateCreate(LocalDateTime.now());
        warningMessageNew.setUpdateBy("system");
        warningMessageNew.setDateUpdate(LocalDateTime.now());
        warningMessageMapper.insertSelective(warningMessageNew);
    }
}
