package com.zoniot.ccrc.service.impl;


import com.zoniot.ccrc.dto.UseWaterDto;
import com.zoniot.ccrc.repository.MongoMeterReadRecord;
import com.zoniot.ccrc.service.MongoMeterReadRecordService;
import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.aggregation.ConditionalOperators;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class MongoMeterReadRecordServiceImpl implements MongoMeterReadRecordService {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public List<MongoMeterReadRecord> findMongoMeterReadRecordList(Query query) {
        return mongoTemplate.find(query, MongoMeterReadRecord.class,"sc_meter_read_record");
    }



    @Override
    public List<UseWaterDto> getRealTimeUseWater(Long deviceId, Integer startDate, Integer endDate) {
        Query query = new Query();
        query.addCriteria(Criteria.where("deviceId").is(deviceId));
        if(startDate!=null && endDate!=null)query.addCriteria(Criteria.where("readDate").gte(startDate).lte(endDate));
        query.with(Sort.by(Sort.Order.asc("dateCreate")));
        List<MongoMeterReadRecord> list =  this.findMongoMeterReadRecordList(query);



        List<UseWaterDto> result = new ArrayList<>();
        if (list != null && list.size() > 0) {
            for (MongoMeterReadRecord record : list) {
                UseWaterDto dto = new UseWaterDto();
                dto.setUseVolume(record.getLastCost().doubleValue());
                dto.setDate(record.getReadDate());
                result.add(dto);
            }
        }
        return result;
    }

    @Override
    public List<UseWaterDto> getUseWaterByMonth(Long deviceId, Integer startDate, Integer endDate) {


        ConditionalOperators.Cond a = null;
        List<AggregationOperation> operations = new ArrayList<>();
        operations.add(Aggregation.match(Criteria.where("deviceId").is(deviceId)));
        operations.add(Aggregation.match(Criteria.where("readDate").gte(startDate).lte(endDate)));

        //new Document("$subtract",Array.asList("$modifiedon", "$createdon"));
        ConditionalOperators.Cond readTimesCond = ConditionalOperators.when(new Criteria("readStatus").is(2)).then(1).otherwise(0);

        operations.add(Aggregation.project("lastCost").
                and("readDate").divide(100).as("readDateTemp")
                );
        operations.add(Aggregation.project("lastCost").
                and("readDateTemp").substring(0,6).as("month")
        );
        operations.add(Aggregation.group("month")
                .sum("lastCost").as("lastCost")
                );
        operations.add(Aggregation.project().
                and("_id").as("date").
                and("lastCost").as("useVolume").andExclude("_id"));
        // 3,聚合查询所有信息
        Aggregation aggregation = Aggregation.newAggregation(operations).withOptions(Aggregation.newAggregationOptions().allowDiskUse(true).build());
        // 4,查询结果
        AggregationResults<UseWaterDto> aggregationResults = mongoTemplate.aggregate(aggregation, "sc_meter_read_record", UseWaterDto.class);
        // 5,获取结果
        List<UseWaterDto> list = aggregationResults.getMappedResults();
        List<UseWaterDto> results = new ArrayList<>();
        if (list != null && list.size() > 0) {
            for (UseWaterDto dto : list) {
                UseWaterDto useWaterDto = new UseWaterDto();
                useWaterDto.setUseVolume(dto.getUseVolume());
                useWaterDto.setDate(dto.getDate());
                results.add(useWaterDto);
            }
        }
        return results;
    }

    @Override
    public Double getDeviceVolume(Long deviceId, Integer startDate, Integer endDate) {
        List<AggregationOperation> operations = new ArrayList<>();
        operations.add(Aggregation.match(Criteria.where("deviceId").is(deviceId)));
        operations.add(Aggregation.match(Criteria.where("readDate").gte(startDate).lte(endDate)));
        operations.add(Aggregation.group()
                .sum("lastCost").as("useVolume"));
        // 3,聚合查询所有信息
        Aggregation aggregation = Aggregation.newAggregation(operations).withOptions(Aggregation.newAggregationOptions().allowDiskUse(true).build());
        // 4,查询结果
        AggregationResults<UseWaterDto> aggregationResults = mongoTemplate.aggregate(aggregation, "sc_meter_read_record", UseWaterDto.class);
        // 5,获取结果
        List<UseWaterDto> list = aggregationResults.getMappedResults();
        if (list != null && list.size() > 0) {
            return list.get(0).getUseVolume();
        }
        return null;
    }

    @Override
    public boolean complyWithWarningRules(Long deviceId, Integer type) {
        boolean flag=false;
        if(type==1){
            Integer date = Integer.valueOf(LocalDateTime.now().plusDays(-1).format(DateTimeFormatter.ofPattern("yyyyMMdd")));
            Query query = new Query();
            query.addCriteria(Criteria.where("deviceId").is(deviceId));
            query.addCriteria(Criteria.where("readDate").is(date));
            List<MongoMeterReadRecord> list =  this.findMongoMeterReadRecordList(query);
            if(!Collections.isEmpty(list)){
                MongoMeterReadRecord mongoMeterReadRecord=list.get(0);
                BigDecimal lastCost = mongoMeterReadRecord.getLastCost();
                BigDecimal bigDecimal=new BigDecimal("2");
                if(lastCost.compareTo(bigDecimal)==1){
                    flag=true;
                }
            }
        }
        return flag;
    }
}
