package com.zoniot.ccrc.service.impl;

import com.alibaba.fastjson.JSON;
import com.zoniot.ccrc.commom.exception.ServiceException;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.OrganizationMapper;
import com.zoniot.ccrc.dao.UserMapper;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.dto.OrganizationDto;
import com.zoniot.ccrc.entity.Organization;
import com.zoniot.ccrc.service.OrganizationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class OrganizationServiceImpl implements OrganizationService {

    @Resource
    private OrganizationMapper organizationMapper;
    @Resource
    private UserMapper userMapper;

    @Override
    public int insert(Organization organization) {
        organization.setStatus(1);
        organization.setCreateBy(UserUtil.getCurrentUser().getUsername());
        organization.setCreateDate(LocalDateTime.now());
        organization.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        organization.setUpdateDate(LocalDateTime.now());
        return organizationMapper.insert(organization);
    }

    @Override
    public int insertSelective(Organization organization) {
        organization.setStatus(1);
        organization.setCreateBy(UserUtil.getCurrentUser().getUsername());
        organization.setCreateDate(LocalDateTime.now());
        organization.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        organization.setUpdateDate(LocalDateTime.now());
        return organizationMapper.insertSelective(organization);
    }

    @Override
    public int insertList(List<Organization> organizations) {
        return organizationMapper.insertList(organizations);
    }

    @Override
    public int updateByPrimaryKeySelective(Organization organization) {
        organization.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        organization.setUpdateDate(LocalDateTime.now());
        return organizationMapper.updateByPrimaryKeySelective(organization);
    }

    @Override
    public List<OrganizationDto> getOrganTree() {
        List<OrganizationDto> list = organizationMapper.findList(UserUtil.getCurrentUser().getSiteId());
        return TreeUtil.getOrganTree(list, 0, 1);
    }

    @Override
    public List<OrganizationDto> findListByCustomer()
    {
        //List<OrganizationDto> list = organizationMapper.findListByCustomer(UserUtil.getCurrentUser().getSiteId(),UserUtil.getCurrentUser().getCustomerId());

        List<OrganizationDto> list = organizationMapper.findListForCustomer(UserUtil.getCurrentUser().getCustomerId());
        return TreeUtil.getOrganTree(list, 0, 1);
    }

    @Override
    public void add(Organization organization) {
        log.info("begin add Organization organization:" + JSON.toJSONString(organization));
        LoginUser loginUser = UserUtil.getCurrentUser();
        int resultName = organizationMapper.findByNameUnique(null,loginUser.getSiteId(), organization.getParentId(), organization.getName());
        if (resultName > 0) {
            throw new ServiceException(-900, "机构名称已存在");
        }

        organization.setSiteId(UserUtil.getCurrentUser().getSiteId());
        if (loginUser.getCustomerId() != null) {
            organization.setCustomerId(BigInteger.valueOf(loginUser.getCustomerId()));
        }

        int result = this.insert(organization);
        organizationMapper.updateCommutity(organization.getCommunityId(),organization.getId());
        log.info("end add Organization result:" + result);

    }

    @Override
    public void edit(Organization organization) {
        log.info("begin edit Organization organization:" + JSON.toJSONString(organization));
        LoginUser loginUser = UserUtil.getCurrentUser();
        int resultName = organizationMapper.findByNameUnique(organization.getId(),loginUser.getSiteId(), organization.getParentId(), organization.getName());
        if (resultName > 0) {
            throw new ServiceException(-900, "机构名称已存在");
        }
        organizationMapper.deleteCommutityRations(organization.getId());
        organizationMapper.updateCommutity(organization.getCommunityId(),organization.getId());

        int result = this.updateByPrimaryKeySelective(organization);
        log.info("end edit Organization result:" + result);
    }

    @Override
    public void delete(Integer id) {
        log.info("begin delete Organization id:" + id);
        Organization organization = new Organization();
        organization.setId(id);
        organization.setStatus(0);
        organizationMapper.deleteCommutityRations(organization.getId());
        int result = this.updateByPrimaryKeySelective(organization);
        log.info("end delete Organization result:" + result);
    }

    @Override
    public Map<String, Integer> deleteQuery(Integer id) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        Map<String, Integer> map = new HashMap<>();

        map.put("childrenNum", organizationMapper.countParentId(id));
        map.put("userNum", userMapper.countOrganId(id));
        return map;
    }
}
