package com.zoniot.ccrc.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.zoniot.ccrc.commom.exception.ServiceException;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.RoleMapper;
import com.zoniot.ccrc.dao.RolePermissionMapper;
import com.zoniot.ccrc.dao.SiteUserMapper;
import com.zoniot.ccrc.dao.UserRoleMapper;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.entity.Role;
import com.zoniot.ccrc.entity.RolePermission;
import com.zoniot.ccrc.service.RoleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

@Service

@Slf4j
public class RoleServiceImpl implements RoleService {

    @Resource
    private RoleMapper roleMapper;
    @Resource
    private RolePermissionMapper rolePermissionMapper;
    @Resource
    private SiteUserMapper siteUserMapper;
    @Resource
    private UserRoleMapper userRoleMapper;


    @Override
    public int insert(Role role) {
        role.setStatus(1);
        role.setCreateBy(UserUtil.getCurrentUser().getUsername());
        role.setCreateDate(LocalDateTime.now());
        role.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        role.setUpdateDate(LocalDateTime.now());
        return roleMapper.insert(role);
    }

    @Override
    public int insertSelective(Role role) {
        role.setStatus(1);
        role.setCreateBy(UserUtil.getCurrentUser().getUsername());
        role.setCreateDate(LocalDateTime.now());
        role.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        role.setUpdateDate(LocalDateTime.now());
        return roleMapper.insertSelective(role);
    }

    @Override
    public int insertList(List<Role> roles) {
        return roleMapper.insertList(roles);
    }

    @Override
    public int update(Role role) {
        role.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        role.setUpdateDate(LocalDateTime.now());
        return roleMapper.updateByPrimaryKeySelective(role);
    }

    @Override
    public Pagination<Role> getRoleList(String name, int pageNum, int pageSize) {
        LoginUser loginUser = UserUtil.getCurrentUser();
        PageHelper.startPage(pageNum, pageSize);
        List<Role> list = roleMapper.getBySiteIdAndType(loginUser.getSiteId(), name, 2);
        Pagination<Role> pageInfo = new Pagination<>(list);
        return pageInfo;
    }

    @Override
    @Transactional
    public void addRole(Role role) {
        log.info("begin addRole Role:" + JSON.toJSONString(role));
        LoginUser loginUser = UserUtil.getCurrentUser();

        int result = roleMapper.findByNameUnique(null,loginUser.getSiteId(), role.getName());
        if (result > 0) {
            throw new ServiceException(-900, "角色名称已经存在");
        }

        role.setSiteId(loginUser.getSiteId());
        role.setType(2);
        int i = this.insertSelective(role);
        //添加概览为角色默认

        RolePermission rolePermission = new RolePermission();
        rolePermission.setRid(role.getId());
        rolePermission.setPid(2);//概览
        rolePermission.setStatus(1);
        rolePermission.setCreateBy("admin");
        rolePermission.setCreateDate(LocalDateTime.now());
        rolePermission.setUpdateBy("admin");
        rolePermission.setUpdateDate(LocalDateTime.now());
        int j = rolePermissionMapper.insert(rolePermission);
        log.info("end addRole result=" + (i + j));
    }

    @Override
    @Transactional
    public void editRole(Role role) {
        log.info("begin editRole Role:" + JSON.toJSONString(role));
        LoginUser loginUser = UserUtil.getCurrentUser();
        int nameResult = roleMapper.findByNameUnique(role.getId(),loginUser.getSiteId(), role.getName());
        if (nameResult > 0) {
            throw new ServiceException(-900, "角色名称已经存在");
        }
        int result = this.update(role);
        log.info("end editRole result=" + result);
    }

    @Override
    @Transactional
    public void delRole(Integer roleId) {
        log.info("begin delRole roleId:" + roleId);
        Role role = roleMapper.findById(roleId);
        if (role != null && role.getType() == 1) {
            throw new ServiceException(-900, "站点管理员角色无法删除");
        }
        int result = userRoleMapper.findByRoleId(roleId);
        if (result > 0) {
            throw new ServiceException(-900, "已有用户绑定该角色，请解除后再删除");
        }
        roleMapper.delRole(roleId, UserUtil.getCurrentUser().getUsername());

        //删除角色的权限关系
        rolePermissionMapper.delRolePermissionByRoleId(roleId, UserUtil.getCurrentUser().getUsername());
    }

    @Override
    public List<Role> getSiteRoleList() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        List<Role> list = roleMapper.getBySiteIdAndType(loginUser.getSiteId(), null, null);
        return list;
    }
}
