package com.zoniot.ccrc.service.impl;

import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.DeviceMapper;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.dto.UseWaterAnalyze;
import com.zoniot.ccrc.dto.UseWaterDto;
import com.zoniot.ccrc.dto.WaterPriceDto;
import com.zoniot.ccrc.service.MongoMeterReadRecordService;
import com.zoniot.ccrc.service.StatisticsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class StatisticsServiceImpl implements StatisticsService {
    @Resource
    private DeviceMapper deviceMapper;
    @Autowired
    private MongoMeterReadRecordService mongoMeterReadRecordService;

    @Override
    public List<UseWaterDto> getRealTimeUseWater(Integer period) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyyMMdd");
        LocalDateTime startDateTime = LocalDateTime.now().plusDays(-period);
        LocalDateTime endDateTime = LocalDateTime.now().plusDays(-1);
        Integer startDate = Integer.valueOf(startDateTime.format(df));
        Integer endDate = Integer.valueOf(endDateTime.format(df));
        Long deviceId=getDeviceId();
        List<UseWaterDto> list =mongoMeterReadRecordService.getRealTimeUseWater(deviceId,startDate,endDate);

        //填充缺失月份数据
        fillDataForCertainMonths(period,list,startDateTime,df,0);
        return list;
    }

    private void fillDataForCertainMonths(Integer period,List<UseWaterDto>list,
                                          LocalDateTime startDateTime, DateTimeFormatter df,Integer type){
        List<UseWaterDto> newList = new ArrayList<>();
        for (int i = 0; i < period; i++) {
            LocalDateTime nextDateTime = startDateTime.plusMonths(i);
            if(type==0){
                nextDateTime = startDateTime.plusDays(i);
            }else if(type==1){
                nextDateTime = startDateTime.plusMonths(i);
            }

            Integer times = Integer.valueOf(nextDateTime.format(df));

            long count = list.stream().filter(ite -> ite.getDate().equals(times)).count();
            if(count == 0){
                UseWaterDto newUseWaterDto = new UseWaterDto();
                newUseWaterDto.setUseVolume(0.0);
                newUseWaterDto.setDate(times);
                newList.add(newUseWaterDto);
            }
        }
        list.addAll(newList);
        //重新按天排序
        list.sort(Comparator.comparing(UseWaterDto::getDate));

    }
    void statisticsOnWaterUsage(UseWaterAnalyze useWaterAnalyze,List<UseWaterDto> list){
        //总数
        double sum = list.stream().mapToDouble(UseWaterDto::getUseVolume).sum();
        useWaterAnalyze.setTotalUseVolume(new BigDecimal(sum).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
        // 平均数
        double average = list.stream().mapToDouble(UseWaterDto::getUseVolume).average().getAsDouble();
        //保留两位小数
        BigDecimal b = new BigDecimal(average);
        useWaterAnalyze.setAverageUseVolume(b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());

        // 最大值
        double max = list.stream().mapToDouble(UseWaterDto::getUseVolume).max().getAsDouble();
        UseWaterDto maxUseWaterDto = list.stream().filter(ite -> ite.getUseVolume().equals(max)).collect(Collectors.toList()).get(0);
        useWaterAnalyze.setMaxUseWater(maxUseWaterDto);

        // 最小值
        double min = list.stream().mapToDouble(UseWaterDto::getUseVolume).min().getAsDouble();
        UseWaterDto mixUseWaterDto = list.stream().filter(ite -> ite.getUseVolume().equals(min)).collect(Collectors.toList()).get(0);
        useWaterAnalyze.setMinUseWater(mixUseWaterDto);

    }
    @Override
    public UseWaterAnalyze getThirtyDaysUseWater() {
        List<UseWaterDto> list = getRealTimeUseWater( 30);
        UseWaterAnalyze useWaterAnalyze = new UseWaterAnalyze();
        useWaterAnalyze.setList(list);
        statisticsOnWaterUsage(useWaterAnalyze,list);
        return useWaterAnalyze;
    }

    @Override
    public UseWaterAnalyze getUseWaterAnalyze() {
        Long deviceId=getDeviceId();
        UseWaterAnalyze useWaterAnalyze = new UseWaterAnalyze();
        //近12月
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyyMMdd");
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMM");
        LocalDateTime startDateTime = LocalDateTime.now().plusMonths(-12).with(TemporalAdjusters.firstDayOfMonth());
        LocalDateTime endDateTime = LocalDateTime.now().plusMonths(-1).with(TemporalAdjusters.lastDayOfMonth());
        Integer startDate = Integer.valueOf(startDateTime.format(df));
        Integer endDate = Integer.valueOf(endDateTime.format(df));
        List<UseWaterDto> list =mongoMeterReadRecordService.getUseWaterByMonth(deviceId,startDate,endDate);
        //填充缺失月份数据
        fillDataForCertainMonths(12,list,startDateTime,dateTimeFormatter,1);
        useWaterAnalyze.setList(list);
        statisticsOnWaterUsage(useWaterAnalyze,list);
        return useWaterAnalyze;
    }

    @Override
    public WaterPriceDto waterPriceQuery(Long deviceId, String date) {
        return null;
    }

    @Override
    public Double getDeviceVolume(Integer startDate, Integer endDate) {
        Long deviceId=getDeviceId();
        return mongoMeterReadRecordService.getDeviceVolume(deviceId, startDate, endDate);
    }
    Long getDeviceId(){
        LoginUser loginClientUser = UserUtil.getCurrentUser();
        Long deviceId= deviceMapper.getDeviceIdByUserNumber(loginClientUser.getId());

        return deviceId;
    }
}
