package com.zoniot.ccrc.service.impl;


import com.zoniot.ccrc.commom.utils.ByteArrayUtils;
import com.zoniot.ccrc.commom.utils.JwtTokenUtil;
import com.zoniot.ccrc.commom.utils.RedisUtil;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.UUID;

/**
 * token存到redis的实现类
 */
@Primary
@Service
public class TokenServiceJWTImpl implements TokenService {
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private JwtTokenUtil jwtTokenUtil;
    @Value("360000")
    private Long expiration;


    @Override
    public String saveToken(LoginUser loginUser) {
        if (loginUser.getLoginToken() == null) {
            loginUser.setLoginToken(UUID.randomUUID().toString());
        }

        //保存redis
        String key = "loginToken:" + loginUser.getLoginToken();
        loginUser.setExpireTime(LocalDateTime.now().plusSeconds(expiration));
        redisUtil.setExpire(key.getBytes(), ByteArrayUtils.objectToBytes(loginUser).get(), expiration);

       /* if (loginUser.getLoginToken() != null) {
            String iotTokenKey = "iotToken:" + loginUser.getLoginToken();
            redisUtil.setExpire(iotTokenKey.getBytes(), loginUser.getIotToken().getBytes(), expiration);
        }*/


        return loginUser.getLoginToken();
    }


    @Override
    public void refresh(LoginUser loginUser) {

    }

    @Override
    public LoginUser getLoginUser(String token) {
        //String loginToken = jwtTokenUtil.getLoginUserToken(token);
        String key = "loginToken:" + token;
        byte[] loginTokenByte = redisUtil.get(key.getBytes());
        if (loginTokenByte != null) {
            LoginUser loginUser = (LoginUser) ByteArrayUtils.bytesToObject(loginTokenByte).get();
            return loginUser;
        }else {
            return null;
        }

    }

    @Override
    public boolean deleteToken(String loginToken) {
        String key = "loginToken:" + loginToken;
        redisUtil.del(key.getBytes());
        return true;
    }
}
